// kchecker.h - BoundsChecker definitions for DriverWorks
//=============================================================================
//
// Compuware Corporation
// NuMega Lab
// 9 Townsend West
// Nashua, NH 03060  USA
//
// Copyright (c) 1998 Compuware Corporation. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States.
//
// U.S. GOVERNMENT RIGHTS-Use, duplication, or disclosure by the
// U.S. Government is subject to restrictions as set forth in 
// Compuware Corporation license agreement and as provided in 
// DFARS 227.7202-1(a) and 227.7202-3(a) (1995), 
// DFARS 252.227-7013(c)(1)(ii)(OCT 1988), FAR 12.212(a) (1995), 
// FAR 52.227-19, or FAR 52.227-14 (ALT III), as applicable.  
// Compuware Corporation.
// 
// This product contains confidential information and trade secrets 
// of Compuware Corporation. Use, disclosure, or reproduction is 
// prohibited without the prior express written permission of Compuware 
// Corporation.
//
//=============================================================================

#ifndef __KCHECKER__
#define __KCHECKER__

// forward declarations
typedef struct _URB* PURB;

// Typedef for struct containing information on calling BCHKD. BCHKD writes all fields.
typedef struct BchkdInfo_t {
	ULONG Version;
	VOID (__cdecl *	EventReportRoutine)(ULONG EventCode, PDRIVER_OBJECT pDriver,...);
	ULONG DriverWorksBaseOrdinal;
	PDRIVER_OBJECT ThisDriver;
	PVOID Reserved2;
	} BCHKD_INFO;

#ifndef  _Kchklib_
// Global BCHKD_INFO structure filled in by BCHKD at init time
// It should only be declared here if this is not a compilation of the 
//  kchecker library 
extern BCHKD_INFO BchkdInfo;
#endif

// Dummy report routine for when BCHKD is not present
VOID __cdecl DummyReportRoutine(ULONG EventCode, PDRIVER_OBJECT pDriver,...);
// Initialization function
NTSTATUS BoundsCheckerInit(PDRIVER_OBJECT);



// ========================================================
// ========================================================

// Ioctl code used by DriverWorks to get BCHKD information

#define FILE_DEVICE_BCHKD  0x00008400
#define IOCTL_BCHKD_GET_REPORT_ENTRY \
	CTL_CODE(FILE_DEVICE_BCHKD, 0x5, METHOD_NEITHER, FILE_ANY_ACCESS)

#define MIN_ALLOWED_BCHKD_VERSION	0x00000100
#define MAX_ALLOWED_BCHKD_VERSION	0x000FFFFE 

// ========================================================
//  Macro BOUNDS_CHECKER
// ========================================================

// The following macros are used in the definition of macro BOUNDS_CHECKER.
// Invocations of BOUNDS_CHECKER are sprinkled throughout the source code
// to send interesting messages to BCHKD. The parameters to macro BOUNDS_CHECKER
// are
//
//	BOUNDS_CHECKER(id, arglist)
//
// where
//	id is the unique string that identifies the event 
//	arglist is the list of arguments, enclosed in parentheses
//
// Example:
//
// 	BOUNDS_CHECKER(MEM_ALLOC_FAILURE, (pDriver, PagedPool, sizeof(_IRP)));
//
// = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = 
// Macro LISTn just removes the parentheses. This is required since macros cannot
// take a variable number of parameters.
//
#define LIST1(a) a
#define LIST2(a,b) a,b
#define LIST3(a,b,c) a, b, c
#define LIST4(a,b,c,d) a, b, c, d
#define LIST5(a,b,c,d,e) a, b, c, d, e
#define LIST6(a,b,c,d,e,f) a, b, c, d, e, f
#define LIST7(a,b,c,d,e,f,g) a, b, c, d, e, f, g
#define LIST8(a,b,c,d,e,f,g,h) a, b, c, d, e, f, g, h

// The following macros are necessary to make the preprocessor do the right thing with the
// symbolic argcount constant.
#define __CAT(x,y) x##y
#define METALIST(n, list) __CAT(LIST,n) list // creates an invocation of LISTn(list)
//
// Macro BC_EVENT_VALUE(id) computes the event id to be passed to BCHKD. The value is computed
// by adding the constant that is #defined in this header to the base ordinal returned by BCHKD
// at initialization.
#define BC_EVENT_VALUE(id)	 	(EVENT_DWORKS_##id + BchkdInfo.DriverWorksBaseOrdinal)

// Macro BC_FIXED_ARGS(id) is the pair of arguments that are always the first two parameters to
// the event reporting function in BCHKD
#define BC_FIXED_ARGS(id) 		BC_EVENT_VALUE(id), BchkdInfo.ThisDriver

// Now we can define BOUNDS_CHECKER. 
#if DBG && !defined(VXD_COMPATLIB)
// Macro BOUNDS_CHECKER calls the entry point returned by BCHKD at initialization. In the case that
// BCHKD is not present, a stub has been substituted. This macro casts the pointer to the entry to a
// function type specific to the event, which is set up by macro EVENT_PARAMETERS below.
#define BOUNDS_CHECKER(id, arglist) \
         (BchkdInfo.EventReportRoutine)(BC_FIXED_ARGS(id), METALIST(ARGCOUNT_##id, arglist))
#else 
// Note that if !DBG, then BOUNDS_CHECKER generates no code
#define BOUNDS_CHECKER(id, arglist)

#endif // DBG

// Macro EVENT_PARAMETERS(id) forms the start of a typedef. This is used to generate prototypes for
// the  reporting function which are customized to each event. That way, we build in type checking
// to ensure that the parameter count and types defined below agree with the actual invocation
// of BOUNDS_CHECKER in the source code.

#define EVENT_PARAMETERS(id) typedef VOID (__cdecl *FUNCTYPE_##id)(ULONG EventId, PDRIVER_OBJECT pDriver,

// ========================================================
// ========================================================
// ========================================================

// The definitions of events sent to BCHKD by DriverWorks begin here.

// ========================================================
#define EVENT_DWORKS_GENERIC_1					0x0000
#define ARGCOUNT_GENERIC_1	4
EVENT_PARAMETERS(GENERIC_1)
		PVOID pObject,
		PCSZ String,
		ULONG Ulong1,
		ULONG Ulong2
		);
// ========================================================
#define EVENT_DWORKS_GENERIC_2					0x0001
#define ARGCOUNT_GENERIC_2 4
EVENT_PARAMETERS(GENERIC_2)
		PVOID pObject,
		PUNICODE_STRING uString,
		ULONG Ulong1,
		ULONG Ulong2
		);

/* template
// ========================================================
#define EVENT_DWORKS_			0x00
#define ARGCOUNT_  n
EVENT_PARAMETERS()
*/

// ===============================================================================
#endif // __KCHECKER__

