// kchecker.c - DriverWorks/BoundsChecker interface
//=============================================================================
//
// Compuware Corporation
// NuMega Lab
// 9 Townsend West
// Nashua, NH 03060  USA
//
// Copyright (c) 1998 Compuware Corporation. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States.
//
// U.S. GOVERNMENT RIGHTS-Use, duplication, or disclosure by the
// U.S. Government is subject to restrictions as set forth in 
// Compuware Corporation license agreement and as provided in 
// DFARS 227.7202-1(a) and 227.7202-3(a) (1995), 
// DFARS 252.227-7013(c)(1)(ii)(OCT 1988), FAR 12.212(a) (1995), 
// FAR 52.227-19, or FAR 52.227-14 (ALT III), as applicable.  
// Compuware Corporation.
// 
// This product contains confidential information and trade secrets 
// of Compuware Corporation. Use, disclosure, or reproduction is 
// prohibited without the prior express written permission of Compuware 
// Corporation.
//
//=============================================================================
#include <DWClass.h>
typedef struct SynchCallCompletionContext
{
	PRKEVENT m_pEvent;
	PIO_COMPLETION_ROUTINE m_Completion;
	PVOID m_CompletionContext;
	NTSTATUS m_Status;
	ULONG    m_Information;
	BOOLEAN m_KeepIrp;
}SynchCallCompletionContext_t;

//Some private function definitions
NTSTATUS CallWaitComplete(BOOLEAN, PULONG);
NTSTATUS SynchCallCompletionRoutine(PDEVICE_OBJECT,PIRP,PVOID);

// Global data
    BCHKD_INFO BchkdInfo = {
					0,
					DummyReportRoutine,
					0,
					NULL,
					NULL};
	
	PIRP                m_Irp;
	PIO_STACK_LOCATION  NextStack;
    PFILE_OBJECT        m_pFileObject;
	PDEVICE_OBJECT      m_pDeviceObject;

#pragma code_seg("INIT")

//=============================================================================
//
// BoundsCheckerInit
//
// Initializes the interface between BCHKD and DriverWorks.
// This must be called at DriverEntry time.
//
// This function is pure DDK so that the user need not include vdw.h
//   just to run this function.

	NTSTATUS BoundsCheckerInit(PDRIVER_OBJECT pDriver)
{
	NTSTATUS status;
	ULONG Information;
	UNICODE_STRING UString;

	// initialize globals
	RtlZeroMemory( &BchkdInfo, sizeof( BCHKD_INFO) );

	// init the BchkdInfo struct with the PDRIVER_OBJECT of this driver
	BchkdInfo.ThisDriver = pDriver;

	// init the Bchkd entry point to the local dummy function
	BchkdInfo.EventReportRoutine = DummyReportRoutine;

	// Create a lower device object to send init request to
		
	RtlInitUnicodeString(&UString, L"\\Device\\Bchkd");
	status = IoGetDeviceObjectPointer(&UString,
		                              FILE_READ_DATA,
									  &m_pFileObject, 
									  &m_pDeviceObject);
	
	//status = Bchkd.ConstructorStatus();
	if ( !NT_SUCCESS(status) )
		return status;

	// Allocate an IRP and initialize it for the call to BCHKD
	
	m_Irp = IoAllocateIrp(m_pDeviceObject->StackSize, FALSE);
	NextStack = IoGetNextIrpStackLocation(m_Irp);
	NextStack->MajorFunction = IRP_MJ_INTERNAL_DEVICE_CONTROL;
	NextStack->MinorFunction = 0;
	NextStack->Parameters.DeviceIoControl.IoControlCode = IOCTL_BCHKD_GET_REPORT_ENTRY;
	NextStack->Parameters.DeviceIoControl.OutputBufferLength;
    m_Irp->UserBuffer = &BchkdInfo;

	// Send the request 
	status = CallWaitComplete(TRUE, &Information);

	// release the IRP
	IoFreeIrp(m_Irp);

	// Test if the call failed or the version is not acceptable
	if ( !NT_SUCCESS(status) || (BchkdInfo.EventReportRoutine == NULL) )
		BchkdInfo.EventReportRoutine = DummyReportRoutine;

	else if ((MIN_ALLOWED_BCHKD_VERSION > BchkdInfo.Version) ||
	         (MAX_ALLOWED_BCHKD_VERSION < BchkdInfo.Version) )
	{
		BchkdInfo.EventReportRoutine = DummyReportRoutine;
		status = STATUS_REVISION_MISMATCH;
	}

	return status;
}
//=============================================================================

#pragma code_seg()

//=============================================================================
//
// DummyReportRoutine
//
// The address of this function is written into BchkdInfo if the initialization
// of the interface to BCHKD fails. This way, the calling code does not have to
// be conditionalized on the success of the initialization.
//
void __cdecl DummyReportRoutine(ULONG EventCode, PDRIVER_OBJECT pDriver,...)
{
	return;
}
//=============================================================================
//
// CallWaitComplete
//
//  Call the BoundsChecker driver and wait for the IRP to complete

NTSTATUS CallWaitComplete(BOOLEAN bKeepIrp, ULONG* pInformation)
{
	SynchCallCompletionContext_t Ctx;
	KEVENT EventObject;
	NTSTATUS status;
	PIO_STACK_LOCATION nextStack = IoGetNextIrpStackLocation(m_Irp);

	// Initialize context structure
    
	KeInitializeEvent(&EventObject, NotificationEvent, FALSE); 
	Ctx.m_pEvent = &EventObject;	
	Ctx.m_Completion = nextStack->CompletionRoutine;
	Ctx.m_CompletionContext = nextStack->Context;
	Ctx.m_KeepIrp = bKeepIrp;

	// Set completion handler, context is addr of context struct
	IoSetCompletionRoutine(m_Irp, 
		(PIO_COMPLETION_ROUTINE)SynchCallCompletionRoutine,
		&Ctx,
		TRUE,
		TRUE,
		TRUE
		);
	// Call the BoundsChecker driver

	status = IoCallDriver(m_pDeviceObject, m_Irp);

	// Wait for completion

	status = KeWaitForSingleObject(&EventObject,Executive,KernelMode,TRUE,NULL);

	// extract information from context structure

	if (pInformation)
		*pInformation = Ctx.m_Information;

	return Ctx.m_Status;
}

///////////// Completion routine /////////////////////

NTSTATUS SynchCallCompletionRoutine(
    IN PDEVICE_OBJECT DeviceObject,
    IN PIRP Irp,
    IN PVOID Context
    )
{
	NTSTATUS status;

	SynchCallCompletionContext_t* Ctx = (SynchCallCompletionContext_t*)Context;

	// Call the original completion routine, if any

	if (Ctx->m_Completion)
		status = Ctx->m_Completion(DeviceObject, Irp, Ctx->m_CompletionContext);

	// otherwise, normal pend bit propagation

	else
	{
		if ((Irp->CurrentLocation <= Irp->StackCount) && Irp->PendingReturned)
			IoMarkIrpPending(Irp);

		if (Ctx->m_KeepIrp)
			status = STATUS_MORE_PROCESSING_REQUIRED;
		else
			status = STATUS_SUCCESS;
	}

	// load up the context structure

	Ctx->m_Information = Irp->IoStatus.Information;
	Ctx->m_Status = Irp->IoStatus.Status;

	// signal the event to release waiting thread

	KeSetEvent(Ctx->m_pEvent, IO_NO_INCREMENT, FALSE);

	return status;
}

